@@ -2,28 +2,31 @@ import argparse
 from tempfile import mkstemp
 import subprocess
 
-from .isa import *
+from .insn import *
 from . import __version__
 
 class MachineDecodeError(Exception):
-    pass
+    def __init__(self, word):
+        self.word = word
+    def __str__(self):
+        return "Invalid instruction word: {:08x}".format(self.word)
 
-def decode(machinecode: int):
-    if machinecode & 0x3 != 3:
+def decode(word: int, variant: Variant=RV32I):
+    if word & 0x3 != 3:
         # compact
         for icls in get_insns(cls=InstructionCType):
-            if icls._match(machinecode):
+            if icls._match(word):
                 i = icls()
-                i.decode(machinecode)
+                i.decode(word)
                 return i
-        raise MachineDecodeError("Cannot decode {:04x}".format(machinecode))
-    opcode = machinecode & 0x7F
-    for icls in get_insns():
-        if icls._opcode == opcode and icls._match(machinecode):
+        raise MachineDecodeError(word)
+    opcode = word & 0x7F
+    for icls in get_insns(variant=variant):
+        if icls.field_opcode.value == opcode and icls.match(word):
             i = icls()
-            i.decode(machinecode)
+            i.decode(word)
             return i
-    raise MachineDecodeError()
+    raise MachineDecodeError(word)
 
 
 def read_from_binary(fname: str, *, stoponerror: bool = False):
