@@ -1,18 +1,32 @@
 from random import randrange
+import sys
 
 from .variant import *
 from .types import Register, RegisterFile, TracePC, TraceIntegerRegister, TraceMemory
 from .program import Program
 
+class TerminateException(Exception):
+    """
+    Exception that signal the termination of the program
+    """
+    def __init__(self, returncode):
+        super().__init__()
+        self.returncode = returncode
+
 # todo: raise exception on memory misalignment
 class State(object):
+    single_regs = []
+
     def __init__(self, variant: Variant):
         self.variant = variant
-        self.intreg = RegisterFile(variant.intregs, 32, {0: 0x0})
+        intregs = 32 if variant.baseint == "I" else 16
+        self.intreg = RegisterFile(intregs, 32, {0: 0x0})
         self.pc = Register(32)
         self.pc_update = Register(32)
         self.memory = Memory()
         self.initialized = True
+        # Used for tracking atomic accesses
+        self.reservations = []
 
     def randomize(self):
         self.intreg.randomize()
@@ -33,7 +47,9 @@ class State(object):
         self.pc.set(pc)
 
     def __setattr__(self, key, value):
-        if key is "pc" and "pc_update" in self.__dict__:
+        if key in self.single_regs and key in self.__dict__:
+            getattr(self, key).update(value)
+        elif key == "pc" and "pc_update" in self.__dict__:
             self.pc_update.set(value)
         else:
             super().__setattr__(key, value)
@@ -41,6 +57,16 @@ class State(object):
     def __str__(self):
         return "{}".format(self.intreg)
 
+    def atomic_acquire(self, address):
+        if not address in self.reservations:
+            self.reservations.append(address)
+
+    def atomic_release(self, address):
+        if address in self.reservations:
+            self.reservations.remove(address)
+
+    def atomic_reserved(self, address):
+        return address in self.reservations
 
 class Memory(object):
     def __init__(self, *, base: int = 0, size: int = 2^32):
@@ -96,19 +122,27 @@ class Memory(object):
 
         self.memory_updates = []
 
+class Environment:
+    def call(self, state: State):
+        pass
 
 class Model(object):
-    def __init__(self, variant: Variant, *, verbose = False):
+    def __init__(self, variant: Variant, *, environment: Environment = None, verbose = False, asm_width=20):
         self.state = State(variant)
+        self.environment = environment if environment is not None else Environment()
         self.verbose = verbose
+        if self.verbose is not False:
+            self.verbose_file = sys.stdout if verbose is True else open(self.verbose, "w")
+        self.asm_tpl = "{{:{}}} | [{{}}]\n".format(asm_width)
 
     def issue(self, insn):
         self.state.pc += 4
         expected_pc = self.state.pc
-        insn.execute(self.state)
+        insn.execute(self)
+
         trace = self.state.changes()
-        if self.verbose:
-            print("{:20} | {}".format(str(insn), trace))
+        if self.verbose is not False:
+            self.verbose_file.write(self.asm_tpl.format(str(insn), ", ".join([str(t) for t in trace])))
         self.state.commit()
         return trace
 
@@ -118,8 +152,11 @@ class Model(object):
         elif not isinstance(insn, list):
             insn = [insn]
 
-        for i in insn:
-            self.issue(i)
+        try:
+            for i in insn:
+                self.issue(i)
+        except TerminateException as ex:
+            assert ex.returncode == 0
 
     def randomize(self):
         self.state.randomize()
