@@ -1,4 +1,4 @@
-from .model import Model, Memory
+from .model import Model, Memory, TerminateException
 
 import struct
 
@@ -22,7 +22,10 @@ class Simulator:
       try:
         self.model.issue(self.program[int(self.model.state.pc)>>2])
         cnt += 1
-      except IndexError as e:
+      except TerminateException as exc:
+        assert exc.returncode == 0
+        return cnt
+      except IndexError as exc:
         return cnt
     return cnt
 
