@@ -1,6 +1,7 @@
 from riscvmodel.program.tests import *
+from riscvmodel.program.tests_atomic import *
 from riscvmodel.model import Model
-from riscvmodel.variant import RV32I
+from riscvmodel.variant import RV32I, RV32A
 
 import pytest
 
@@ -21,3 +22,11 @@ def test_model_addi():
     m = Model(RV32I)
     m.execute(pgm)
     check_model(m, pgm.expects())
+
+
+def test_model_lr_sc():
+    """Test if load-reserved and store-conditional sequence works"""
+    pgm = LRSCTest()
+    m   = Model(RV32A)
+    m.execute(pgm)
+    check_model(m, pgm.expects())
\ No newline at end of file
